# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pyyaledoorman']

package_data = \
{'': ['*']}

install_requires = \
['aiohttp>=3.7.4,<4.0.0', 'traitlets>=5.0.5,<6.0.0']

setup_kwargs = {
    'name': 'pyyaledoorman',
    'version': '1.0.9',
    'description': 'Python library for the Yale Doorman Smart API',
    'long_description': 'Pyyaledoorman\n=============\n\n|PyPI| |Python Version| |License|\n\n|Read the Docs| |Tests| |Codecov|\n\n|pre-commit| |Black|\n\n.. |PyPI| image:: https://img.shields.io/pypi/v/pyyaledoorman.svg\n   :target: https://pypi.org/project/pyyaledoorman/\n   :alt: PyPI\n.. |Python Version| image:: https://img.shields.io/pypi/pyversions/pyyaledoorman\n   :target: https://pypi.org/project/pyyaledoorman\n   :alt: Python Version\n.. |License| image:: https://img.shields.io/pypi/l/pyyaledoorman\n   :target: https://opensource.org/licenses/MIT\n   :alt: License\n.. |Read the Docs| image:: https://img.shields.io/readthedocs/pyyaledoorman/latest.svg?label=Read%20the%20Docs\n   :target: https://pyyaledoorman.readthedocs.io/\n   :alt: Read the documentation at https://pyyaledoorman.readthedocs.io/\n.. |Tests| image:: https://github.com/espenfjo/pyyaledoorman/workflows/Tests/badge.svg\n   :target: https://github.com/espenfjo/pyyaledoorman/actions?workflow=Tests\n   :alt: Tests\n.. |Codecov| image:: https://codecov.io/gh/espenfjo/pyyaledoorman/branch/main/graph/badge.svg\n   :target: https://codecov.io/gh/espenfjo/pyyaledoorman\n   :alt: Codecov\n.. |pre-commit| image:: https://img.shields.io/badge/pre--commit-enabled-brightgreen?logo=pre-commit&logoColor=white\n   :target: https://github.com/pre-commit/pre-commit\n   :alt: pre-commit\n.. |Black| image:: https://img.shields.io/badge/code%20style-black-000000.svg\n   :target: https://github.com/psf/black\n   :alt: Black\n\nA Python library for interfacing with the Yale Doorman Home API.\n\nFeatures\n--------\n\n* Read whether lock is unlocked, locked, or door open\n* Locking / Unlocking\n* Enabling / Disabling auto lock\n* Reading Volume level, Language, Auto lock disabled/enabled\n\n\n\nRequirements\n------------\n\n* Yale Doorman home user account.\n\n\nInstallation\n------------\n\nYou can install *Pyyaledoorman* via pip_ from PyPI_:\n\n.. code:: console\n\n   $ pip install pyyaledoorman\n\n\nUsage\n-----\n\nPlease see `Usage <Usage_>`_ for details.\n\n.. code-example\n.. code-block:: python\n\n    import pyyaledoorman\n    import aiohttp\n    import asyncio\n    import pyyaledoorman\n\n\n    async def main():\n        async with aiohttp.ClientSession() as session:\n            client = pyyaledoorman.Client(\n                "username",\n                "password",\n                session=session,\n            )\n            assert await client.login() == True\n            await client.update_devices()\n\n            for device in client.devices:\n                print(device.name)\n                await device.disable_autolock()\n                await device.unlock(pincode="123456")\n\n\n    loop = asyncio.get_event_loop()\n    loop.run_until_complete(main())\n\n.. code-example-end\n\nContributing\n------------\n\nContributions are very welcome.\nTo learn more, see the `Contributor Guide`_.\n\n\nLicense\n-------\n\nDistributed under the terms of the `MIT license`_,\n*Pyyaledoorman* is free and open source software.\n\n\nIssues\n------\n\nIf you encounter any problems,\nplease `file an issue`_ along with a detailed description.\n\n\nCredits\n-------\n\nThis project was generated from `@cjolowicz`_\'s `Hypermodern Python Cookiecutter`_ template.\n\n.. _@cjolowicz: https://github.com/cjolowicz\n.. _Cookiecutter: https://github.com/audreyr/cookiecutter\n.. _MIT license: https://opensource.org/licenses/MIT\n.. _PyPI: https://pypi.org/\n.. _Hypermodern Python Cookiecutter: https://github.com/cjolowicz/cookiecutter-hypermodern-python\n.. _file an issue: https://github.com/espenfjo/pyyaledoorman/issues\n.. _pip: https://pip.pypa.io/\n.. github-only\n.. _Contributor Guide: CONTRIBUTING.rst\n.. _Usage: https://pyyaledoorman.readthedocs.io/en/latest/usage.html\n',
    'author': 'Espen Fjellvær Olsen',
    'author_email': 'espen@mrfjo.org',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/espenfjo/pyyaledoorman',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9.0,<4.0.0',
}


setup(**setup_kwargs)



"""Yale API Client. Used to log in to the API and instantiate Devices."""
import logging
from datetime import datetime
from http.client import FORBIDDEN
from http.client import UNAUTHORIZED
from typing import Any
from typing import Dict
from typing import List
from typing import Optional

from aiohttp import ClientSession

from .const import BASE_URL
from .const import INITIAL_TOKEN
from .const import STATUS_CODES
from .device import Device

_LOGGER = logging.getLogger(__name__)


class AuthenticationError(Exception):
    """Exception for authentication errors."""

    def __init__(self, *args: Any) -> None:
        """Initialize the Exception."""
        Exception.__init__(self, *args)


class Client:
    """Yale Doorman client.

    Arguments:
        username: Username for logging in to the Yale API.
        password: Password for logging in to the Yale API.
        initial_token: Initial token for logging in to the Yale API.
        session (optional): aiohttp ClientSession to use.
    """

    def __init__(
        self,
        username: str,
        password: str,
        initial_token: str = INITIAL_TOKEN,
        session: Optional[ClientSession] = None,
    ) -> None:
        """Initialize the Yale Doorman Client."""
        self.username = username
        self.password = password
        self.initial_token = initial_token
        self.logged_in = datetime.timestamp(datetime.now())
        _LOGGER.info("Logged in to Yale")
        if session:
            self._session = session
        else:
            self._session = ClientSession()
        self._devices: List[Device] = []
        self._token: Optional[str] = None
        self._refresh_token: Optional[str] = None
        self.headers: Dict[str, str] = {}

    @property
    def login_ts(self) -> float:
        """Return the login timestamp."""
        return self._login_ts

    @login_ts.setter
    def login_ts(self, timestamp: float) -> None:
        """Set the login timestamp.

        Arguments:
            timestamp: Timestamp of whence the API was logged in to.
        """
        self._login_ts = timestamp

    @property
    def token(self) -> Optional[str]:
        """Return the access token."""
        return self._token

    @token.setter
    def token(self, _token: str) -> None:
        """Set the access token."""
        self._token = _token
        self.headers["Authorization"] = f"Bearer {self.token}"

    @property
    def refresh_token(self) -> Optional[str]:
        """Return the refresh token."""
        return self._refresh_token

    @refresh_token.setter
    def refresh_token(self, _refresh_token: Optional[str]) -> None:
        """Set the refresh token."""
        self._refresh_token = _refresh_token

    @property
    def token_expires_in(self) -> int:
        """Return seconds from login whence the access token expires."""
        return self._token_expires_in

    @token_expires_in.setter
    def token_expires_in(self, expires: int) -> None:
        """Set number of seconds since login when the access token expires."""
        self._token_expires_in = expires

    @property
    def devices(self) -> List[Device]:
        """Return list of Devices."""
        return self._devices

    @property
    def session(self) -> ClientSession:
        """Return the aiohttp session."""
        return self._session

    async def login(self) -> bool:  # raises: AuthenticationError
        """Log in to the Yale API.

        Returns:
            bool: True if successfully logged in.

        Raises:
            AuthenticationError: Supplied username, password or initial token is wrong.
        """
        _LOGGER.info("Trying to log in to Yale..")
        headers = {
            "Accept": "application/json",
            "Authorization": f"Basic {self.initial_token}",
        }
        if self.refresh_token:
            auth_data = {
                "grant_type": "refresh_token",
                "refresh_token": self.refresh_token,
            }
        else:
            auth_data = {
                "grant_type": "password",
                "username": self.username,
                "password": self.password,
            }
        async with self._session.post(
            f"{BASE_URL}/o/token/",
            data=auth_data,
            headers=headers,
        ) as resp:
            status = resp.status
            data = await resp.json()

            if status in (FORBIDDEN, UNAUTHORIZED):
                if self.refresh_token:
                    self.refresh_token = None
                    return await self.login()
                _LOGGER.debug(
                    "Failed to authenticate with Yale Smart Alarm. Error: %s", data
                )
                raise AuthenticationError(
                    "Failed to authenticate with Yale Smart Alarm. Check credentials."
                )
            self.token_expires_in = data.get("expires_in")

            self.login_ts = datetime.timestamp(datetime.now())
            self.token = data.get("access_token")
            self.refresh_token = data.get("refresh_token")
            return True

    async def _validate_access_token(self) -> None:
        """Verify that our access token is still valid."""
        now = datetime.now()
        timestamp = datetime.timestamp(now)
        # Check if login time + expires in is past now(-ish)
        if (self.login_ts + self.token_expires_in) <= (timestamp - 1000):
            await self.login()

    async def update_devices(self) -> None:
        """Update the device states."""
        await self._validate_access_token()
        url = f"{BASE_URL}/api/panel/device_status/"
        async with self._session.get(
            url, headers=self.headers, raise_for_status=False
        ) as resp:
            res = await resp.json()
            if res.get("code") == STATUS_CODES["SUCCESS"]:
                for device in res.get("data"):
                    existing = False
                    for existing_device in self._devices:
                        if existing_device.device_id == device.get("device_id"):
                            existing = True
                            await existing_device.update_state()
                            break
                    if existing:
                        continue

                    self._devices.append(Device(self, device))
            else:
                _LOGGER.debug("Couldn't fetch devices. Unknown error!")
